@@ -45,9 +45,7 @@ extern string old_filters;
 
 extern Xapian::Database db;
 extern Xapian::Enquire * enquire;
-extern Xapian::RSet rset;
 
-extern Xapian::docid topdoc;
 extern Xapian::docid hits_per_page;
 extern Xapian::docid min_hits;
 
@@ -64,8 +62,6 @@ extern bool collapse;
 
 extern map<string, string> option;
 
-extern const string default_dbname;
-
 extern bool set_content_type, suppress_http_headers;
 
 #endif // OMEGA_INCLUDED_OMEGA_H
